@@ -13,6 +13,7 @@ import (
 
 	"github.com/percona/percona-backup-mongodb/pbm"
 	plog "github.com/percona/percona-backup-mongodb/pbm/log"
+	"github.com/percona/percona-backup-mongodb/version"
 )
 
 func backup(cn *pbm.PBM, bcpName, compression string) (string, error) {
@@ -60,33 +61,22 @@ func backup(cn *pbm.PBM, bcpName, compression string) (string, error) {
 		return "", err
 	}
 
-	storeString := ""
-	switch cfg.Storage.Type {
-	case pbm.StorageS3:
-		storeString = "s3://"
-		if cfg.Storage.S3.EndpointURL != "" {
-			storeString += cfg.Storage.S3.EndpointURL + "/"
-		}
-		storeString += cfg.Storage.S3.Bucket
-		if cfg.Storage.S3.Prefix != "" {
-			storeString += "/" + cfg.Storage.S3.Prefix
-		}
-	case pbm.StorageFilesystem:
-		storeString = cfg.Storage.Filesystem.Path
-	}
-	return storeString, nil
+	return cfg.Storage.Path(), nil
 }
 
-func waitForBcpStatus(ctx context.Context, cn *pbm.PBM, bcpName string) error {
+func waitForBcpStatus(ctx context.Context, cn *pbm.PBM, bcpName string) (err error) {
 	tk := time.NewTicker(time.Second * 1)
 	defer tk.Stop()
-	var err error
-	bmeta := new(pbm.BackupMeta)
+
+	var bmeta *pbm.BackupMeta
 	for {
 		select {
 		case <-tk.C:
 			fmt.Print(".")
 			bmeta, err = cn.GetBackupMeta(bcpName)
+			if errors.Is(err, pbm.ErrNotFound) {
+				continue
+			}
 			if err != nil {
 				return errors.Wrap(err, "get backup metadata")
 			}
@@ -104,6 +94,9 @@ func waitForBcpStatus(ctx context.Context, cn *pbm.PBM, bcpName string) error {
 				return errors.New(bmeta.Error + rs)
 			}
 		case <-ctx.Done():
+			if bmeta == nil {
+				return errors.New("no progress from leader, backup metadata not found")
+			}
 			rs := ""
 			for _, s := range bmeta.Replsets {
 				rs += fmt.Sprintf("- Backup on replicaset \"%s\" in state: %v\n", s.Name, s.Status)
@@ -147,6 +140,9 @@ func printBackupList(cn *pbm.PBM, size int64) {
 		if b.Status != pbm.StatusDone {
 			continue
 		}
+		if !version.Compatible(version.DefaultInfo.Version, b.PBMVersion) {
+			continue
+		}
 
 		fmt.Printf("  %s [complete: %s]\n", b.Name, fmtTS(int64(b.LastWriteTS.T)))
 	}
@@ -198,6 +194,7 @@ func bcpMatchCluster(bcps []pbm.BackupMeta, shards []pbm.Shard, confsrv string)
 	}
 }
 
+// printPITR shows only chunks derived from `Done` and compatible version's backups
 func printPITR(cn *pbm.PBM, size int, full bool) {
 	on, err := cn.IsPITR()
 	if err != nil {
@@ -248,6 +245,16 @@ func printPITR(cn *pbm.PBM, size int, full bool) {
 			fmt.Println("\nPITR <off>:")
 		}
 		for _, tl := range pbm.MergeTimelines(rstlines...) {
+			bcp, err := cn.GetLastBackup(&primitive.Timestamp{T: tl.End, I: 0})
+			if err != nil {
+				log.Printf("ERROR: get backup for timeline: %s", tl)
+				continue
+			}
+			if bcp == nil ||
+				bcp.Status != pbm.StatusDone ||
+				!version.Compatible(version.DefaultInfo.Version, bcp.PBMVersion) {
+				continue
+			}
 			fmt.Println(" ", tl)
 		}
 	}
