@@ -1,20 +1,25 @@
 package main
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"log"
+	"net/url"
 	"os"
 	"strings"
 	"time"
 
 	"github.com/pkg/errors"
+	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/options"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
 	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/pitr"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
+	"github.com/percona/percona-backup-mongodb/version"
 )
 
 type statusSect struct {
@@ -38,14 +43,18 @@ const (
 	formatJSON
 )
 
-func status(cn *pbm.PBM, f outFormat) {
+func status(cn *pbm.PBM, curi string, f outFormat) {
 	var o []statusSect
 
 	sections := []struct {
 		name string
 		f    func(cn *pbm.PBM) (fmt.Stringer, error)
 	}{
-		{"Cluster", clusterStatus},
+		{"Cluster",
+			func(cn *pbm.PBM) (fmt.Stringer, error) {
+				return clusterStatus(cn, curi)
+			},
+		},
 		{"PITR incremental backup", getPitrStatus},
 		{"Currently running", getCurrOps},
 		{"Backups", getStorageStat},
@@ -184,50 +193,35 @@ func (c cluster) String() (s string) {
 	return s
 }
 
-func clusterStatus(cn *pbm.PBM) (fmt.Stringer, error) {
-	inf, err := cn.GetNodeInfo()
+func clusterStatus(cn *pbm.PBM, uri string) (fmt.Stringer, error) {
+	clstr, err := cn.ClusterMembers(nil)
 	if err != nil {
-		return nil, errors.Wrap(err, "get cluster info")
-	}
-
-	type clstr struct {
-		rs    string
-		nodes []string
-	}
-	topology := []clstr{
-		{
-			rs:    inf.SetName,
-			nodes: inf.Hosts,
-		},
-	}
-
-	if inf.IsSharded() {
-		shrd, err := cn.GetShards()
-		if err != nil {
-			return nil, errors.Wrap(err, "get cluster shards")
-		}
-		for _, s := range shrd {
-			topology = append(topology, clstr{
-				rs:    s.RS,
-				nodes: strings.Split(strings.TrimPrefix(s.Host, s.RS+"/"), ","),
-			})
-		}
-	}
-
-	err = cn.AgentStatusGC()
-	if err != nil {
-		return nil, errors.Wrap(err, "clean-up stale agent statuses")
+		return nil, errors.Wrap(err, "get cluster members")
 	}
 
 	var ret cluster
-	for _, shrd := range topology {
-		lrs := rs{Name: shrd.rs}
-		for i, n := range shrd.nodes {
-			lrs.Nodes = append(lrs.Nodes, node{Host: shrd.rs + "/" + n})
+
+	for _, c := range clstr {
+		rconn, err := connect(cn.Context(), uri, c.Host)
+		if err != nil {
+			return nil, errors.Wrapf(err, "connect to `%s` [%s]", c.RS, c.Host)
+		}
+
+		sstat, sterr := pbm.GetReplsetStatus(cn.Context(), rconn)
+
+		// don't need the connection anymore despite the result
+		rconn.Disconnect(cn.Context())
+
+		if sterr != nil {
+			return nil, errors.Wrapf(err, "get replset status for `%s`", c.RS)
+		}
+		lrs := rs{Name: c.RS}
+		for i, n := range sstat.Members {
+			lrs.Nodes = append(lrs.Nodes, node{Host: c.RS + "/" + n.Name})
 
 			nd := &lrs.Nodes[i]
 
-			stat, err := cn.GetAgentStatus(shrd.rs, n)
+			stat, err := cn.GetAgentStatus(c.RS, n.Name)
 			if errors.Is(err, mongo.ErrNoDocuments) {
 				nd.Ver = "NOT FOUND"
 				continue
@@ -244,13 +238,55 @@ func clusterStatus(cn *pbm.PBM) (fmt.Stringer, error) {
 	return ret, nil
 }
 
+func connect(ctx context.Context, uri, hosts string) (*mongo.Client, error) {
+	var host string
+	chost := strings.Split(hosts, "/")
+	if len(chost) > 1 {
+		host = chost[1]
+	} else {
+		host = chost[0]
+	}
+
+	curi, err := url.Parse("mongodb://" + strings.Replace(uri, "mongodb://", "", 1))
+	if err != nil {
+		return nil, errors.Wrapf(err, "parse mongo-uri '%s'", uri)
+	}
+
+	// Preserving the `replicaSet` parameter will cause an error while connecting to the ConfigServer (mismatched replicaset names)
+	query := curi.Query()
+	query.Del("replicaSet")
+	curi.RawQuery = query.Encode()
+	curi.Host = host
+
+	conn, err := mongo.NewClient(options.Client().ApplyURI(curi.String()).SetAppName("pbm-status"))
+	if err != nil {
+		return nil, errors.Wrap(err, "create mongo client")
+	}
+	err = conn.Connect(ctx)
+	if err != nil {
+		return nil, errors.Wrap(err, "connect")
+	}
+
+	err = conn.Ping(ctx, nil)
+	if err != nil {
+		return nil, errors.Wrap(err, "ping")
+	}
+
+	return conn, nil
+}
+
 type pitrStat struct {
-	Status string `json:"status"`
-	Err    string `json:"error,omitempty"`
+	InConf  bool   `json:"conf"`
+	Running bool   `json:"run"`
+	Err     string `json:"error,omitempty"`
 }
 
 func (p pitrStat) String() string {
-	s := fmt.Sprintf("Status [%s]", p.Status)
+	status := "OFF"
+	if p.InConf || p.Running {
+		status = "ON"
+	}
+	s := fmt.Sprintf("Status [%s]", status)
 	if p.Err != "" {
 		s += fmt.Sprintf("\n! ERROR while running PITR backup: %s", p.Err)
 	}
@@ -259,15 +295,16 @@ func (p pitrStat) String() string {
 
 func getPitrStatus(cn *pbm.PBM) (fmt.Stringer, error) {
 	var p pitrStat
-	on, err := cn.IsPITR()
+	var err error
+	p.InConf, err = cn.IsPITR()
 	if err != nil {
-		return p, errors.Wrap(err, "unable check PITR status")
+		return p, errors.Wrap(err, "unable check PITR config status")
 	}
-	if !on {
-		p.Status = "OFF"
-		return p, nil
+
+	p.Running, err = cn.PITRrun()
+	if err != nil {
+		return p, errors.Wrap(err, "unable check PITR running status")
 	}
-	p.Status = "ON"
 
 	p.Err, err = getPitrErr(cn)
 
@@ -419,22 +456,24 @@ func getCurrOps(cn *pbm.PBM) (fmt.Stringer, error) {
 }
 
 type storageStat struct {
-	Type     pbm.StorageType `json:"type"`
-	Path     string          `json:"path"`
-	Region   string          `json:"region,omitempty"`
-	Snapshot []snapshotStat  `json:"snapshot"`
-	PITR     []pitrRange     `json:"pitrChunks,omitempty"`
+	Type     string         `json:"type"`
+	Path     string         `json:"path"`
+	Region   string         `json:"region,omitempty"`
+	Snapshot []snapshotStat `json:"snapshot"`
+	PITR     []pitrRange    `json:"pitrChunks,omitempty"`
 }
 
 type snapshotStat struct {
-	Name    string     `json:"name"`
-	Size    int64      `json:"size"`
-	Status  pbm.Status `json:"status"`
-	Err     string     `json:"error,omitempty"`
-	StateTS int64      `json:"completeTS"`
+	Name       string     `json:"name"`
+	Size       int64      `json:"size"`
+	Status     pbm.Status `json:"status"`
+	Err        string     `json:"error,omitempty"`
+	StateTS    int64      `json:"completeTS"`
+	PBMVersion string     `json:"pbmVersion"`
 }
 
 type pitrRange struct {
+	Err   string `json:"error,omitempty"`
 	Range struct {
 		Start int64 `json:"start"`
 		End   int64 `json:"end"`
@@ -443,29 +482,30 @@ type pitrRange struct {
 }
 
 func (s storageStat) String() string {
-	typ := "S3"
-	if s.Type == pbm.StorageFilesystem {
-		typ = "FS"
-	}
-	ret := fmt.Sprintf("%s %s %s\n", typ, s.Region, s.Path)
+	ret := fmt.Sprintf("%s %s %s\n", s.Type, s.Region, s.Path)
 	if len(s.Snapshot) == 0 {
 		return ret + "  (none)"
 	}
 
 	ret += fmt.Sprintln("  Snapshots:")
 	for _, sn := range s.Snapshot {
-		ret += fmt.Sprintf("    %s %s", sn.Name, fmtSize(sn.Size))
+		var status string
 		switch sn.Status {
 		case pbm.StatusDone:
-			ret += fmt.Sprintf(" [complete: %s]", fmtTS(sn.StateTS))
+			status = fmt.Sprintf(" [complete: %s]", fmtTS(sn.StateTS))
 		case pbm.StatusCancelled:
-			ret += fmt.Sprintf(" [!cancelled: %s]", fmtTS(sn.StateTS))
+			status = fmt.Sprintf(" [!cancelled: %s]", fmtTS(sn.StateTS))
 		case pbm.StatusError:
-			ret += fmt.Sprintf(" [ERROR: %s] [%s]", sn.Err, fmtTS(sn.StateTS))
+			status = fmt.Sprintf(" [ERROR: %s] [%s]", sn.Err, fmtTS(sn.StateTS))
 		default:
-			ret += fmt.Sprintf(" [running: %s / %s]", sn.Status, fmtTS(sn.StateTS))
+			status = fmt.Sprintf(" [running: %s / %s]", sn.Status, fmtTS(sn.StateTS))
 		}
-		ret += "\n"
+
+		var v string
+		if !version.Compatible(version.DefaultInfo.Version, sn.PBMVersion) {
+			v = fmt.Sprintf(" !!! backup v%s is not compatible with PBM v%s", sn.PBMVersion, version.DefaultInfo.Version)
+		}
+		ret += fmt.Sprintf("    %s %s%s%s\n", sn.Name, fmtSize(sn.Size), status, v)
 	}
 
 	if len(s.PITR) == 0 {
@@ -475,7 +515,11 @@ func (s storageStat) String() string {
 	ret += fmt.Sprintln("  PITR chunks:")
 
 	for _, sn := range s.PITR {
-		ret += fmt.Sprintf("    %s - %s %s\n", fmtTS(sn.Range.Start), fmtTS(sn.Range.End), fmtSize(sn.Size))
+		var v string
+		if sn.Err != "" {
+			v = fmt.Sprintf(" !!! %s", sn.Err)
+		}
+		ret += fmt.Sprintf("    %s - %s %s%s\n", fmtTS(sn.Range.Start), fmtTS(sn.Range.End), fmtSize(sn.Size), v)
 	}
 
 	return ret
@@ -489,23 +533,12 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 		return s, errors.Wrap(err, "get config")
 	}
 
-	s.Type = cfg.Storage.Type
+	s.Type = cfg.Storage.Typ()
 
-	switch cfg.Storage.Type {
-	case pbm.StorageS3:
-		var url []string
-		if cfg.Storage.S3.EndpointURL != "" {
-			url = append(url, cfg.Storage.S3.EndpointURL)
-		}
-		url = append(url, cfg.Storage.S3.Bucket)
-		if cfg.Storage.S3.Prefix != "" {
-			url = append(url, cfg.Storage.S3.Prefix)
-		}
-		s.Path = strings.Join(url, "/")
+	if cfg.Storage.Type == pbm.StorageS3 {
 		s.Region = cfg.Storage.S3.Region
-	case pbm.StorageFilesystem:
-		s.Path = cfg.Storage.Filesystem.Path
 	}
+	s.Path = cfg.Storage.Path()
 
 	bcps, err := cn.BackupsList(0)
 	if err != nil {
@@ -526,16 +559,17 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 	// which the `confsrv` param in `bcpMatchCluster` is all about
 	bcpMatchCluster(bcps, shards, inf.SetName)
 
-	stg, err := cn.GetStorage(nil)
+	stg, err := cn.GetStorage(cn.Logger().NewEvent("", "", "", primitive.Timestamp{}))
 	if err != nil {
 		return s, errors.Wrap(err, "get storage")
 	}
 
 	for _, bcp := range bcps {
 		snpsht := snapshotStat{
-			Name:    bcp.Name,
-			Status:  bcp.Status,
-			StateTS: bcp.LastTransitionTS,
+			Name:       bcp.Name,
+			Status:     bcp.Status,
+			StateTS:    bcp.LastTransitionTS,
+			PBMVersion: bcp.PBMVersion,
 		}
 
 		switch bcp.Status {
@@ -567,7 +601,7 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage) (pr []pitrRange, err error)
 		return pr, errors.Wrap(err, "get cluster members")
 	}
 
-	fl, err := stg.List(pbm.PITRfsPrefix)
+	fl, err := stg.List(pbm.PITRfsPrefix, "")
 	if err != nil {
 		return pr, errors.Wrap(err, "get chunks list")
 	}
@@ -595,6 +629,17 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage) (pr []pitrRange, err error)
 		rng.Range.Start = int64(tl.Start)
 		rng.Range.End = int64(tl.End)
 		rng.Size = tl.Size
+
+		bcp, err := cn.GetLastBackup(&primitive.Timestamp{T: tl.End, I: 0})
+		if err != nil {
+			log.Printf("ERROR: get backup for timeline: %s", tl)
+			continue
+		}
+		if bcp == nil {
+			rng.Err = "no backup found"
+		} else if !version.Compatible(version.DefaultInfo.Version, bcp.PBMVersion) {
+			rng.Err = fmt.Sprintf("backup v%s is not compatible with PBM v%s", bcp.PBMVersion, version.DefaultInfo.Version)
+		}
 		pr = append(pr, rng)
 	}
 
