@@ -22,12 +22,18 @@ func NewMongoPBM(ctx context.Context, connectionURI string) (*MongoPBM, error) {
 		return nil, errors.Wrap(err, "connect")
 	}
 
+	cn.InitLogger("", "")
+
 	return &MongoPBM{
 		p:   cn,
 		ctx: ctx,
 	}, nil
 }
 
+func (m *MongoPBM) SendCmd(cmd pbm.Cmd) error {
+	return m.p.SendCmd(cmd)
+}
+
 func (m *MongoPBM) BackupsList(limit int64) ([]pbm.BackupMeta, error) {
 	return m.p.BackupsList(limit)
 }
