@@ -20,6 +20,7 @@ type Cluster struct {
 	mongos   *pbm.Mongo
 	shards   map[string]*pbm.Mongo
 	mongopbm *pbm.MongoPBM
+	confsrv  string
 }
 
 type ClusterConf struct {
@@ -37,6 +38,7 @@ func New(cfg ClusterConf) *Cluster {
 		mongos:   mgoConn(ctx, cfg.Mongos),
 		mongopbm: pbmConn(ctx, cfg.Configsrv),
 		shards:   make(map[string]*pbm.Mongo),
+		confsrv:  cfg.ConfigsrvRsName,
 	}
 	for name, uri := range cfg.Shards {
 		c.shards[name] = mgoConn(ctx, uri)
@@ -159,13 +161,13 @@ func (c *Cluster) BackupWaitDone(bcpName string) {
 	log.Printf("backup finished '%s'\n", bcpName)
 }
 
-func (c *Cluster) GenerateBallastData(amount int) {
-	log.Println("generating ballast data")
-	err := c.mongos.GenBallast(amount)
+func (c *Cluster) SetBallastData(amount int64) {
+	log.Println("set ballast data to", amount)
+	cnt, err := c.mongos.SetBallast(amount)
 	if err != nil {
 		log.Fatalln("generating ballast:", err)
 	}
-	log.Println("ballast data generated")
+	log.Println("ballast data:", cnt)
 }
 
 func (c *Cluster) DataChecker() (check func()) {
@@ -207,8 +209,11 @@ func (c *Cluster) checkBackup(bcpName string, waitFor time.Duration) error {
 			return errors.Errorf("timeout reached. pbm status:\n%s", sts)
 		case <-tkr.C:
 			m, err := c.mongopbm.GetBackupMeta(bcpName)
+			if errors.Is(err, pbmt.ErrNotFound) {
+				continue
+			}
 			if err != nil {
-				errors.Wrap(err, "get backup meta")
+				return errors.Wrap(err, "get backup meta")
 			}
 			switch m.Status {
 			case pbmt.StatusDone:
