@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
-	"fmt"
 	"io"
 	"log"
 	"time"
@@ -21,7 +20,6 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm"
 	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
-	"github.com/percona/percona-backup-mongodb/pbm/storage/s3"
 	"github.com/percona/percona-backup-mongodb/version"
 )
 
@@ -36,53 +34,58 @@ func init() {
 type Backup struct {
 	cn   *pbm.PBM
 	node *pbm.Node
-	ctx  context.Context
-	opid string
 }
 
-func New(ctx context.Context, cn *pbm.PBM, node *pbm.Node) *Backup {
+func New(cn *pbm.PBM, node *pbm.Node) *Backup {
 	return &Backup{
 		cn:   cn,
 		node: node,
-		ctx:  ctx,
 	}
 }
 
 // Run runs the backup
-func (b *Backup) Run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
-	return b.run(bcp, opid, l)
+func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
+	return b.run(ctx, bcp, opid, l)
 }
 
-type errMetaExists struct {
-	error
-}
+func (b *Backup) Init(bcp pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMode) error {
+	meta := &pbm.BackupMeta{
+		OPID:           opid.String(),
+		Name:           bcp.Name,
+		Compression:    bcp.Compression,
+		StartTS:        time.Now().Unix(),
+		Status:         pbm.StatusStarting,
+		Replsets:       []pbm.BackupReplset{},
+		LastWriteTS:    primitive.Timestamp{T: 1, I: 1}, // the driver (mongo?) sets TS to the current wall clock if TS was 0, so have to init with 1
+		PBMVersion:     version.DefaultInfo.Version,
+		Nomination:     []pbm.BackupRsNomination{},
+		BalancerStatus: balancer,
+	}
 
-func (e *errMetaExists) Error() string {
-	return fmt.Sprintf("write backup meta to db: %v", e.error)
-}
+	cfg, err := b.cn.GetConfig()
+	if err == pbm.ErrStorageUndefined {
+		return errors.New("backups cannot be saved because PBM storage configuration hasn't been set yet")
+	} else if err != nil {
+		return errors.Wrap(err, "unable to get PBM config settings")
+	}
+	meta.Store = cfg.Storage
 
-func (e *errMetaExists) Unwrap() error { return e.error }
+	ver, err := b.node.GetMongoVersion()
+	if err == nil {
+		meta.MongoVersion = ver.VersionString
+	}
+
+	return b.cn.SetBackupMeta(meta)
+}
 
 // run the backup.
 // TODO: describe flow
-func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
+func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
 	inf, err := b.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get cluster info")
 	}
 
-	b.opid = opid.String()
-	meta := &pbm.BackupMeta{
-		OPID:        b.opid,
-		Name:        bcp.Name,
-		StartTS:     time.Now().Unix(),
-		Compression: bcp.Compression,
-		Status:      pbm.StatusStarting,
-		Replsets:    []pbm.BackupReplset{},
-		LastWriteTS: primitive.Timestamp{T: 1, I: 1}, // (andrew) I dunno why, but the driver (mongo?) sets TS to the current wall clock if TS was 0, so have to init with 1
-		PBMVersion:  version.DefaultInfo.Version,
-	}
-
 	rsName := inf.SetName
 
 	rsMeta := pbm.BackupReplset{
@@ -99,62 +102,56 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 	if err != nil {
 		return errors.Wrap(err, "unable to get PBM storage configuration settings")
 	}
+
+	bcpm, err := b.cn.GetBackupMeta(bcp.Name)
+	if err != nil {
+		return errors.Wrap(err, "balancer status, get backup meta")
+	}
+
 	// on any error the RS' and the backup' (in case this is the backup leader) meta will be marked aproprietly
 	defer func() {
 		if err != nil {
-			// protection from the lagging leader node.
-			// if the leader node wasn't able to create meta (it's prjbably already exists)
-			// it shouldn't try change anything, otherwise it may commit errors state to the
-			// meta of already done backup. see https://jira.percona.com/browse/PBM-520
-			if _, ok := err.(*errMetaExists); ok {
-				return
-			}
-
 			status := pbm.StatusError
 			if errors.Is(err, ErrCancelled) {
 				status = pbm.StatusCancelled
 
-				meta.Status = pbm.StatusCancelled
-				meta.Replsets = append(meta.Replsets, rsMeta)
+				meta := &pbm.BackupMeta{
+					Name:     bcp.Name,
+					Status:   pbm.StatusCancelled,
+					Replsets: []pbm.BackupReplset{rsMeta},
+				}
+
 				l.Info("delete artefacts from storage: %v", b.cn.DeleteBackupFiles(meta, stg))
 			}
 
-			// protection from the lagging node.
-			// during the StatusStarting stage a meta doesn't created yet,
-			// so the node can mark with the error only meta of already finished backup
-			// this is what we want to prevent. see https://jira.percona.com/browse/PBM-520
-			if meta.Status != pbm.StatusStarting {
-				ferr := b.cn.ChangeRSState(bcp.Name, rsMeta.Name, status, err.Error())
-				l.Info("mark RS as %s `%v`: %v", status, err, ferr)
-			}
+			ferr := b.cn.ChangeRSState(bcp.Name, rsMeta.Name, status, err.Error())
+			l.Info("mark RS as %s `%v`: %v", status, err, ferr)
+
 			if inf.IsLeader() {
 				ferr := b.cn.ChangeBackupState(bcp.Name, status, err.Error())
 				l.Info("mark backup as %s `%v`: %v", status, err, ferr)
 			}
 		}
-	}()
 
-	ver, err := b.node.GetMongoVersion()
-	if err == nil {
-		meta.MongoVersion = ver.VersionString
-	}
-
-	cfg, err := b.cn.GetConfig()
-	if err == pbm.ErrStorageUndefined {
-		return errors.New("backups cannot be saved because PBM storage configuration hasn't been set yet")
-	} else if err != nil {
-		return errors.Wrap(err, "unable to get PBM config settings")
-	}
-	meta.Store = cfg.Storage
-	// Erase credentials data
-	meta.Store.S3.Credentials = s3.Credentials{}
-
-	if inf.IsLeader() {
-		err = b.cn.SetBackupMeta(meta)
-		if err != nil {
-			return &errMetaExists{err}
+		// Turn the balancer back on if needed
+		//
+		// Every agent will check if the balancer was on before the backup started.
+		// And will try to turn it on again if so. So if the leader node went down after turning off
+		// the balancer some other node will bring it back.
+		// TODO: what if all agents went down.
+		if bcpm.BalancerStatus != pbm.BalancerModeOn {
+			return
 		}
 
+		err = b.cn.SetBalancerStatus(pbm.BalancerModeOn)
+		if err != nil {
+			l.Error("set balancer ON: %v", err)
+			return
+		}
+		l.Debug("set balancer on")
+	}()
+
+	if inf.IsLeader() {
 		hbstop := make(chan struct{})
 		defer close(hbstop)
 		go func() {
@@ -172,23 +169,23 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 				}
 			}
 		}()
+
+		if bcpm.BalancerStatus == pbm.BalancerModeOn {
+			err = b.cn.SetBalancerStatus(pbm.BalancerModeOff)
+			if err != nil {
+				return errors.Wrap(err, "set balancer OFF")
+			}
+			l.Debug("set balancer off")
+		}
 	}
 
 	// Waiting for StatusStarting to move further.
 	// In case some preparations has to be done before backup.
-	err = b.waitForStatus(bcp.Name, pbm.StatusStarting)
+	err = b.waitForStatus(bcp.Name, pbm.StatusStarting, &pbm.WaitBackupStart)
 	if err != nil {
 		return errors.Wrap(err, "waiting for start")
 	}
 
-	// A fallback. If there are some leftovers and main node
-	// hasn't cleaned up it yet we should stop and notify user
-	// since backup would be unrestorable.
-	err = b.node.EnsureNoTMPcoll()
-	if err != nil {
-		return errors.Wrap(err, "EnsureNoTMPcoll")
-	}
-
 	rsMeta.Status = pbm.StatusRunning
 	err = b.cn.AddRSMeta(bcp.Name, rsMeta)
 	if err != nil {
@@ -196,7 +193,7 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 	}
 
 	if inf.IsLeader() {
-		err := b.reconcileStatus(bcp.Name, pbm.StatusRunning, inf, &pbm.WaitBackupStart)
+		err := b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusRunning, inf, &pbm.WaitBackupStart)
 		if err != nil {
 			if errors.Cause(err) == errConvergeTimeOut {
 				return errors.Wrap(err, "couldn't get response from all shards")
@@ -206,9 +203,9 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 	}
 
 	// Waiting for cluster's StatusRunning to move further.
-	err = b.waitForStatus(bcp.Name, pbm.StatusRunning)
+	err = b.waitForStatus(bcp.Name, pbm.StatusRunning, nil)
 	if err != nil {
-		return errors.Wrap(err, "waiting for start")
+		return errors.Wrap(err, "waiting for running")
 	}
 
 	oplog := NewOplog(b.node)
@@ -222,6 +219,31 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 		return errors.Wrap(err, "set shard's first write ts")
 	}
 
+	// Save users and roles to the tmp collections so the restore would copy that data
+	// to the system collections. Have to do this because of issues with the restore and preserverUUID.
+	// see: https://jira.percona.com/browse/PBM-636 and comments
+	lw, err := b.node.CopyUsersNRolles()
+	if err != nil {
+		return errors.Wrap(err, "copy users and roles for the restore")
+	}
+
+	defer func() {
+		l.Info("dropping tmp collections")
+		err := b.node.DropTMPcoll()
+		if err != nil {
+			l.Warning("drop tmp users and roles: %v", err)
+		}
+	}()
+
+	// before proceeding any further we have to be sure that tmp users and roles
+	// have replicated to the node we're about to take a backup from
+	// *copying made on a primary but backup does a secondary node
+	l.Debug("wait for tmp users %v", lw)
+	err = b.node.WaitForWrite(lw)
+	if err != nil {
+		return errors.Wrap(err, "wait for tmp users and roles replication")
+	}
+
 	sz, err := b.node.SizeDBs()
 	if err != nil {
 		return errors.Wrap(err, "mongodump")
@@ -234,7 +256,7 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 	}
 
 	dump := newDump(b.node.ConnURI(), b.node.DumpConns())
-	_, err = Upload(b.ctx, dump, stg, bcp.Compression, rsMeta.DumpName, sz)
+	_, err = Upload(ctx, dump, stg, bcp.Compression, rsMeta.DumpName, sz)
 	if err != nil {
 		return errors.Wrap(err, "mongodump")
 	}
@@ -256,30 +278,31 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 	}
 
 	if inf.IsLeader() {
-		err := b.reconcileStatus(bcp.Name, pbm.StatusDumpDone, inf, nil)
+		err := b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusDumpDone, inf, nil)
 		if err != nil {
 			return errors.Wrap(err, "check cluster for dump done")
 		}
 
-		err = b.setClusterLastWrite(bcp.Name)
+		err = b.setClusterFirtsLastWrite(bcp.Name)
 		if err != nil {
-			return errors.Wrap(err, "set cluster last write ts")
+			return errors.Wrap(err, "set cluster first & last write ts")
 		}
 	}
 
-	err = b.waitForStatus(bcp.Name, pbm.StatusDumpDone)
+	err = b.waitForStatus(bcp.Name, pbm.StatusDumpDone, nil)
 	if err != nil {
 		return errors.Wrap(err, "waiting for dump done")
 	}
 
-	lwTS, err := b.waitForLastWrite(bcp.Name)
+	fwTS, lwTS, err := b.waitForFirstLastWrite(bcp.Name)
 	if err != nil {
-		return errors.Wrap(err, "waiting and reading cluster last write ts")
+		return errors.Wrap(err, "get cluster first & last write ts")
 	}
 
-	oplog.SetTailingSpan(oplogTS, lwTS)
+	l.Debug("set oplog span to %v / %v", fwTS, lwTS)
+	oplog.SetTailingSpan(fwTS, lwTS)
 	// size -1 - we're assuming oplog never exceed 97Gb (see comments in s3.Save method)
-	_, err = Upload(b.ctx, oplog, stg, bcp.Compression, rsMeta.OplogName, -1)
+	_, err = Upload(ctx, oplog, stg, bcp.Compression, rsMeta.OplogName, -1)
 	if err != nil {
 		return errors.Wrap(err, "oplog")
 	}
@@ -289,7 +312,7 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 	}
 
 	if inf.IsLeader() {
-		err = b.reconcileStatus(bcp.Name, pbm.StatusDone, inf, nil)
+		err = b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusDone, inf, nil)
 		if err != nil {
 			return errors.Wrap(err, "check cluster for backup done")
 		}
@@ -301,7 +324,7 @@ func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error
 	}
 
 	// to be sure the locks released only after the "done" status had written
-	err = b.waitForStatus(bcp.Name, pbm.StatusDone)
+	err = b.waitForStatus(bcp.Name, pbm.StatusDone, nil)
 	return errors.Wrap(err, "waiting for done")
 }
 
@@ -309,22 +332,6 @@ const maxReplicationLagTimeSec = 21
 
 // NodeSuits checks if node can perform backup
 func NodeSuits(node *pbm.Node, inf *pbm.NodeInfo) (bool, error) {
-	if inf.IsStandalone() {
-		return false, errors.New("mongod node can not be used to fetch a consistent backup because it has no oplog. Please restart it as a primary in a single-node replicaset to make it compatible with PBM's backup method using the oplog")
-	}
-
-	// for the cases when no secondary was good enough for backup or there are no secondaries alive
-	// wait for 90% of WaitBackupStart and then try to acquire a lock.
-	// by that time healthy secondaries should have already acquired a lock.
-	//
-	// but no need to wait if this is the only node (the single-node replica set).
-	//
-	// TODO ? there is still a chance that the lock gonna be stolen from the healthy secondary node
-	// TODO ? (due tmp network issues node got the command later than the primary, but it's maybe for the good that the node with the faulty network doesn't start the backup)
-	if inf.IsPrimary && inf.Me == inf.Primary && len(inf.Hosts) > 1 {
-		time.Sleep(pbm.WaitBackupStart * 9 / 10)
-	}
-
 	status, err := node.Status()
 	if err != nil {
 		return false, errors.Wrap(err, "get node status")
@@ -411,26 +418,26 @@ func Upload(ctx context.Context, src Source, dst storage.Storage, compression pb
 	return n, nil
 }
 
-func (b *Backup) reconcileStatus(bcpName string, status pbm.Status, ninf *pbm.NodeInfo, timeout *time.Duration) error {
+func (b *Backup) reconcileStatus(bcpName, opid string, status pbm.Status, ninf *pbm.NodeInfo, timeout *time.Duration) error {
 	shards, err := b.cn.ClusterMembers(ninf)
 	if err != nil {
 		return errors.Wrap(err, "get cluster members")
 	}
 
 	if timeout != nil {
-		return errors.Wrap(b.convergeClusterWithTimeout(bcpName, shards, status, *timeout), "convergeClusterWithTimeout")
+		return errors.Wrap(b.convergeClusterWithTimeout(bcpName, opid, shards, status, *timeout), "convergeClusterWithTimeout")
 	}
-	return errors.Wrap(b.convergeCluster(bcpName, shards, status), "convergeCluster")
+	return errors.Wrap(b.convergeCluster(bcpName, opid, shards, status), "convergeCluster")
 }
 
 // convergeCluster waits until all given shards reached `status` and updates a cluster status
-func (b *Backup) convergeCluster(bcpName string, shards []pbm.Shard, status pbm.Status) error {
+func (b *Backup) convergeCluster(bcpName, opid string, shards []pbm.Shard, status pbm.Status) error {
 	tk := time.NewTicker(time.Second * 1)
 	defer tk.Stop()
 	for {
 		select {
 		case <-tk.C:
-			ok, err := b.converged(bcpName, shards, status)
+			ok, err := b.converged(bcpName, opid, shards, status)
 			if err != nil {
 				return err
 			}
@@ -446,7 +453,7 @@ func (b *Backup) convergeCluster(bcpName string, shards []pbm.Shard, status pbm.
 var errConvergeTimeOut = errors.New("reached converge timeout")
 
 // convergeClusterWithTimeout waits up to the geiven timeout until all given shards reached `status` and then updates the cluster status
-func (b *Backup) convergeClusterWithTimeout(bcpName string, shards []pbm.Shard, status pbm.Status, t time.Duration) error {
+func (b *Backup) convergeClusterWithTimeout(bcpName, opid string, shards []pbm.Shard, status pbm.Status, t time.Duration) error {
 	tk := time.NewTicker(time.Second * 1)
 	defer tk.Stop()
 	tout := time.NewTicker(t)
@@ -454,7 +461,7 @@ func (b *Backup) convergeClusterWithTimeout(bcpName string, shards []pbm.Shard,
 	for {
 		select {
 		case <-tk.C:
-			ok, err := b.converged(bcpName, shards, status)
+			ok, err := b.converged(bcpName, opid, shards, status)
 			if err != nil {
 				return err
 			}
@@ -469,7 +476,7 @@ func (b *Backup) convergeClusterWithTimeout(bcpName string, shards []pbm.Shard,
 	}
 }
 
-func (b *Backup) converged(bcpName string, shards []pbm.Shard, status pbm.Status) (bool, error) {
+func (b *Backup) converged(bcpName, opid string, shards []pbm.Shard, status pbm.Status) (bool, error) {
 	shardsToFinish := len(shards)
 	bmeta, err := b.cn.GetBackupMeta(bcpName)
 	if err != nil {
@@ -487,7 +494,7 @@ func (b *Backup) converged(bcpName string, shards []pbm.Shard, status pbm.Status
 				// check if node alive
 				lock, err := b.cn.GetLockData(&pbm.LockHeader{
 					Type:    pbm.CmdBackup,
-					OPID:    b.opid,
+					OPID:    opid,
 					Replset: shard.Name,
 				})
 
@@ -526,13 +533,22 @@ func (b *Backup) converged(bcpName string, shards []pbm.Shard, status pbm.Status
 	return false, nil
 }
 
-func (b *Backup) waitForStatus(bcpName string, status pbm.Status) error {
+func (b *Backup) waitForStatus(bcpName string, status pbm.Status, waitFor *time.Duration) error {
+	var tout <-chan time.Time
+	if waitFor != nil {
+		tmr := time.NewTimer(*waitFor)
+		defer tmr.Stop()
+		tout = tmr.C
+	}
 	tk := time.NewTicker(time.Second * 1)
 	defer tk.Stop()
 	for {
 		select {
 		case <-tk.C:
 			bmeta, err := b.cn.GetBackupMeta(bcpName)
+			if errors.Is(err, pbm.ErrNotFound) {
+				continue
+			}
 			if err != nil {
 				return errors.Wrap(err, "get backup metadata")
 			}
@@ -554,13 +570,15 @@ func (b *Backup) waitForStatus(bcpName string, status pbm.Status) error {
 			case pbm.StatusError:
 				return errors.Errorf("cluster failed: %v", err)
 			}
+		case <-tout:
+			return errors.New("no backup meta, looks like a leader failed to start")
 		case <-b.cn.Context().Done():
 			return nil
 		}
 	}
 }
 
-func (b *Backup) waitForLastWrite(bcpName string) (primitive.Timestamp, error) {
+func (b *Backup) waitForFirstLastWrite(bcpName string) (first, last primitive.Timestamp, err error) {
 	tk := time.NewTicker(time.Second * 1)
 	defer tk.Stop()
 	for {
@@ -568,13 +586,23 @@ func (b *Backup) waitForLastWrite(bcpName string) (primitive.Timestamp, error) {
 		case <-tk.C:
 			bmeta, err := b.cn.GetBackupMeta(bcpName)
 			if err != nil {
-				return primitive.Timestamp{}, errors.Wrap(err, "get backup metadata")
+				return first, last, errors.Wrap(err, "get backup metadata")
 			}
-			if bmeta.LastWriteTS.T > 0 {
-				return bmeta.LastWriteTS, nil
+
+			clusterTime, err := b.cn.ClusterTime()
+			if err != nil {
+				return first, last, errors.Wrap(err, "read cluster time")
+			}
+
+			if bmeta.Hb.T+pbm.StaleFrameSec < clusterTime.T {
+				return first, last, errors.Errorf("backup stuck, last beat ts: %d", bmeta.Hb.T)
+			}
+
+			if bmeta.FirstWriteTS.T > 0 && bmeta.LastWriteTS.T > 0 {
+				return bmeta.FirstWriteTS, bmeta.LastWriteTS, nil
 			}
 		case <-b.cn.Context().Done():
-			return primitive.Timestamp{}, nil
+			return first, last, nil
 		}
 	}
 }
@@ -598,20 +626,24 @@ func writeMeta(stg storage.Storage, meta *pbm.BackupMeta) error {
 	return errors.Wrap(err, "write to store")
 }
 
-func (b *Backup) setClusterLastWrite(bcpName string) error {
+func (b *Backup) setClusterFirtsLastWrite(bcpName string) error {
 	bmeta, err := b.cn.GetBackupMeta(bcpName)
 	if err != nil {
 		return errors.Wrap(err, "get backup metadata")
 	}
 
-	lw := primitive.Timestamp{}
+	var fw, lw primitive.Timestamp
 	for _, rs := range bmeta.Replsets {
 		if primitive.CompareTimestamp(lw, rs.LastWriteTS) == -1 {
 			lw = rs.LastWriteTS
 		}
+
+		if fw.T == 0 || primitive.CompareTimestamp(fw, rs.FirstWriteTS) == 1 {
+			fw = rs.FirstWriteTS
+		}
 	}
 
-	err = b.cn.SetLastWrite(bcpName, lw)
+	err = b.cn.SetFirstLastWrite(bcpName, fw, lw)
 	return errors.Wrap(err, "set timestamp")
 }
 
