@@ -1,6 +1,7 @@
 package pbm
 
 import (
+	"fmt"
 	"reflect"
 	"strconv"
 	"strings"
@@ -15,6 +16,7 @@ import (
 
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
+	"github.com/percona/percona-backup-mongodb/pbm/storage/azure"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/blackhole"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/fs"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/s3"
@@ -25,6 +27,7 @@ type Config struct {
 	PITR    PITRConf            `bson:"pitr" json:"pitr" yaml:"pitr"`
 	Storage StorageConf         `bson:"storage" json:"storage" yaml:"storage"`
 	Restore RestoreConf         `bson:"restore" json:"restore,omitempty" yaml:"restore,omitempty"`
+	Backup  BackupConf          `bson:"backup" json:"backup,omitempty" yaml:"backup,omitempty"`
 	Epoch   primitive.Timestamp `bson:"epoch" json:"-" yaml:"-"`
 }
 
@@ -39,6 +42,7 @@ type StorageType string
 const (
 	StorageUndef      StorageType = ""
 	StorageS3         StorageType = "s3"
+	StorageAzure      StorageType = "azure"
 	StorageFilesystem StorageType = "filesystem"
 	StorageBlackHole  StorageType = "blackhole"
 )
@@ -47,15 +51,61 @@ const (
 type StorageConf struct {
 	Type       StorageType `bson:"type" json:"type" yaml:"type"`
 	S3         s3.Conf     `bson:"s3,omitempty" json:"s3,omitempty" yaml:"s3,omitempty"`
+	Azure      azure.Conf  `bson:"azure,omitempty" json:"azure,omitempty" yaml:"azure,omitempty"`
 	Filesystem fs.Conf     `bson:"filesystem,omitempty" json:"filesystem,omitempty" yaml:"filesystem,omitempty"`
 }
 
+func (s *StorageConf) Typ() string {
+	switch s.Type {
+	case StorageS3:
+		return "S3"
+	case StorageAzure:
+		return "Azure"
+	case StorageFilesystem:
+		return "FS"
+	case StorageBlackHole:
+		return "BlackHole"
+	default:
+		return "Unknown"
+	}
+}
+
+func (s *StorageConf) Path() string {
+	path := ""
+	switch s.Type {
+	case StorageS3:
+		path = "s3://"
+		if s.S3.EndpointURL != "" {
+			path += s.S3.EndpointURL + "/"
+		}
+		path += s.S3.Bucket
+		if s.S3.Prefix != "" {
+			path += "/" + s.S3.Prefix
+		}
+	case StorageAzure:
+		path = fmt.Sprintf(azure.BlobURL, s.Azure.Account, s.Azure.Container)
+		if s.Azure.Prefix != "" {
+			path += "/" + s.Azure.Prefix
+		}
+	case StorageFilesystem:
+		path = s.Filesystem.Path
+	case StorageBlackHole:
+		path = "BlackHole"
+	}
+
+	return path
+}
+
 // RestoreConf is config options for the restore
 type RestoreConf struct {
 	BatchSize           int `bson:"batchSize" json:"batchSize,omitempty" yaml:"batchSize,omitempty"` // num of documents to buffer
 	NumInsertionWorkers int `bson:"numInsertionWorkers" json:"numInsertionWorkers,omitempty" yaml:"numInsertionWorkers,omitempty"`
 }
 
+type BackupConf struct {
+	Priority map[string]float64 `bson:"priority,omitempty" json:"priority,omitempty" yaml:"priority,omitempty"`
+}
+
 type confMap map[string]reflect.Kind
 
 // _confmap is a list of config's valid keys and its types
@@ -231,6 +281,9 @@ func (p *PBM) GetConfigYaml(fieldRedaction bool) ([]byte, error) {
 		if c.Storage.S3.Credentials.Vault.Token != "" {
 			c.Storage.S3.Credentials.Vault.Token = "***"
 		}
+		if c.Storage.Azure.Credentials.Key != "" {
+			c.Storage.Azure.Credentials.Key = "***"
+		}
 	}
 
 	b, err := yaml.Marshal(c)
@@ -261,6 +314,8 @@ func (p *PBM) GetStorage(l *log.Event) (storage.Storage, error) {
 	switch c.Storage.Type {
 	case StorageS3:
 		return s3.New(c.Storage.S3, l)
+	case StorageAzure:
+		return azure.New(c.Storage.Azure, l)
 	case StorageFilesystem:
 		return fs.New(c.Storage.Filesystem), nil
 	case StorageBlackHole:
