@@ -25,7 +25,6 @@ type IBackup struct {
 	rs     string
 	span   time.Duration
 	lastTS primitive.Timestamp
-	log    *pbm.Logger
 }
 
 // NewBackup creates an incremental backup object
@@ -35,7 +34,6 @@ func NewBackup(rs string, cn *pbm.PBM, node *pbm.Node) *IBackup {
 		node: node,
 		rs:   rs,
 		span: pbm.PITRdefaultSpan,
-		log:  node.Log,
 	}
 	if ibackupspan != "" {
 		s, err := strconv.Atoi(ibackupspan)
@@ -96,12 +94,17 @@ func (e ErrOpMoved) Error() string {
 	return fmt.Sprintf("pitr slicing resumed on node %s", e.to)
 }
 
+// LogStartMsg message to log on successful streaming start
+const LogStartMsg = "start_ok"
+
 // Stream streaming (saving) chunks of the oplog to the given storage
-func (i *IBackup) Stream(ctx context.Context, wakeupSig <-chan struct{}, to storage.Storage, compression pbm.CompressionType) error {
+func (i *IBackup) Stream(ctx context.Context, ep pbm.Epoch, wakeupSig <-chan struct{}, to storage.Storage, compression pbm.CompressionType) error {
 	if i.lastTS.T == 0 {
 		return errors.New("no starting point defined")
 	}
-	i.log.Info(pbm.CmdPITR, "", "streaming started from %v / %v", time.Unix(int64(i.lastTS.T), 0).UTC(), i.lastTS.T)
+	l := i.pbm.Logger().NewEvent(string(pbm.CmdPITR), "", "", ep.TS())
+	l.Debug(LogStartMsg)
+	l.Info("streaming started from %v / %v", time.Unix(int64(i.lastTS.T), 0).UTC(), i.lastTS.T)
 
 	tk := time.NewTicker(i.span)
 	defer tk.Stop()
@@ -122,14 +125,25 @@ func (i *IBackup) Stream(ctx context.Context, wakeupSig <-chan struct{}, to stor
 		// wrapping up at the current point-in-time
 		// upload the chunks up to the current time and return
 		case <-ctx.Done():
-			i.log.Info(pbm.CmdPITR, "", "got done signal, stopping")
+			l.Info("got done signal, stopping")
 			lastSlice = true
 		// on wakeup or tick whatever comes first do the job
 		case <-wakeupSig:
-			i.log.Info(pbm.CmdPITR, "", "got wake_up signal")
+			l.Info("got wake_up signal")
 		case <-tk.C:
 		}
 
+		// if it's last slice, epoch probably already changed (e.g. due to config changes) and it's ok
+		if !lastSlice {
+			cep, err := i.pbm.GetEpoch()
+			if err != nil {
+				return errors.Wrap(err, "get epoch")
+			}
+			if primitive.CompareTimestamp(ep.TS(), cep.TS()) != 0 {
+				return errors.Errorf("epoch mismatch. Got sleep in %v, woke up in %v. Too old for that stuff.", ep.TS(), cep.TS())
+			}
+		}
+
 		nextChunkT := time.Now().Add(i.span)
 
 		// check if the node is still any good to make backups
@@ -165,6 +179,16 @@ func (i *IBackup) Stream(ctx context.Context, wakeupSig <-chan struct{}, to stor
 			return errors.Wrap(err, "check lock")
 		}
 
+		// in case there is a lock, even legit (our own, or backup's one) but it is stale
+		// we sould return so the slicer whould get thru the lock aquisition again.
+		ts, err := i.pbm.ClusterTime()
+		if err != nil {
+			return errors.Wrap(err, "read cluster time")
+		}
+		if ld.Heartbeat.T+pbm.StaleFrameSec < ts.T {
+			return errors.Errorf("stale lock %#v, last beat ts: %d", ld.LockHeader, ld.Heartbeat.T)
+		}
+
 		switch ld.Type {
 		case pbm.CmdPITR:
 			if ld.Node != nodeInfo.Me {
@@ -175,7 +199,7 @@ func (i *IBackup) Stream(ctx context.Context, wakeupSig <-chan struct{}, to stor
 				return errors.Wrap(err, "define last write timestamp")
 			}
 		case pbm.CmdBackup:
-			sliceTo, err = i.backupStartTS(ld.BackupName)
+			sliceTo, err = i.backupStartTS(ld.OPID)
 			if err != nil {
 				return errors.Wrap(err, "get backup start TS")
 			}
@@ -191,6 +215,15 @@ func (i *IBackup) Stream(ctx context.Context, wakeupSig <-chan struct{}, to stor
 		// if use parent ctx, upload will be canceled on the "done" signal
 		_, err = backup.Upload(context.Background(), oplog, to, compression, fname, -1)
 		if err != nil {
+			// PITR chunks have no metadata to indicate any failed state and if something went
+			// wrong during the data read we may end up with an already created file. Although
+			// the failed range won't be saved in db as the available for restore. It would get
+			// in there after the storage resync. see: https://jira.percona.com/browse/PBM-602
+			l.Debug("remove %s due to upload errors", fname)
+			derr := to.Delete(fname)
+			if derr != nil {
+				l.Error("remove %s: %v", fname, derr)
+			}
 			return errors.Wrapf(err, "unable to upload chunk %v.%v", i.lastTS.T, sliceTo.T)
 		}
 
@@ -210,10 +243,10 @@ func (i *IBackup) Stream(ctx context.Context, wakeupSig <-chan struct{}, to stor
 		if !lastSlice {
 			logm += fmt.Sprintf(". Next chunk creation scheduled to begin at ~%s", nextChunkT.Format("2006-01-02T15:04:05"))
 		}
-		i.log.Info(pbm.CmdPITR, "", logm)
+		l.Info(logm)
 
 		if lastSlice {
-			i.log.Info(pbm.CmdPITR, "", "pausing/stopping with last_ts %v", time.Unix(int64(sliceTo.T), 0).UTC())
+			l.Info("pausing/stopping with last_ts %v", time.Unix(int64(sliceTo.T), 0).UTC())
 			return nil
 		}
 
@@ -242,11 +275,11 @@ func (i *IBackup) getOpLock(l *pbm.LockHeader) (ld pbm.LockData, err error) {
 	return ld, nil
 }
 
-func (i *IBackup) backupStartTS(bcp string) (ts primitive.Timestamp, err error) {
+func (i *IBackup) backupStartTS(opid string) (ts primitive.Timestamp, err error) {
 	tk := time.NewTicker(time.Second)
 	defer tk.Stop()
 	for j := 0; j < int(pbm.WaitBackupStart.Seconds()); j++ {
-		b, err := i.pbm.GetBackupMeta(bcp)
+		b, err := i.pbm.GetBackupByOPID(opid)
 		if err != nil {
 			return ts, errors.Wrap(err, "get backup meta")
 		}
