@@ -18,6 +18,7 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
+	"github.com/percona/percona-backup-mongodb/version"
 )
 
 func init() {
@@ -36,9 +37,14 @@ var excludeFromRestore = []string{
 	pbm.DB + "." + pbm.LogCollection,
 	pbm.DB + "." + pbm.ConfigCollection,
 	pbm.DB + "." + pbm.BcpCollection,
+	pbm.DB + "." + pbm.BcpOldCollection,
 	pbm.DB + "." + pbm.RestoresCollection,
 	pbm.DB + "." + pbm.LockCollection,
+	pbm.DB + "." + pbm.LockOpCollection,
 	pbm.DB + "." + pbm.PITRChunksCollection,
+	pbm.DB + "." + pbm.PITRChunksOldCollection,
+	pbm.DB + "." + pbm.AgentsStatusCollection,
+	pbm.DB + "." + pbm.PBMOpLogCollection,
 	"config.version",
 	"config.mongos",
 	"config.lockpings",
@@ -242,6 +248,9 @@ func (r *Restore) PreparePITR(ts int64) (err error) {
 	if err != nil {
 		return errors.Wrap(err, "define last backup")
 	}
+	if r.bcp == nil {
+		return errors.Errorf("no backup found before ts %v", lastChunk.EndTS)
+	}
 
 	err = r.prepareChunks(r.bcp.LastWriteTS, lastChunk.StartTS)
 	if err != nil {
@@ -279,7 +288,7 @@ func (r *Restore) prepareChunks(from, to primitive.Timestamp) error {
 
 func (r *Restore) PrepareBackup(backupName string) (err error) {
 	r.bcp, err = r.cn.GetBackupMeta(backupName)
-	if errors.Cause(err) == mongo.ErrNoDocuments {
+	if errors.Is(err, pbm.ErrNotFound) {
 		r.bcp, err = r.getMetaFromStore(backupName)
 	}
 
@@ -307,6 +316,10 @@ func (r *Restore) prepareSnapshot() (err error) {
 		return errors.Errorf("backup wasn't successful: status: %s, error: %s", r.bcp.Status, r.bcp.Error)
 	}
 
+	if !version.Compatible(version.DefaultInfo.Version, r.bcp.PBMVersion) {
+		return errors.Errorf("backup version (v%s) is not compatible with PBM v%s", r.bcp.PBMVersion, version.DefaultInfo.Version)
+	}
+
 	if r.nodeInfo.IsLeader() {
 		s, err := r.cn.ClusterMembers(r.nodeInfo)
 		if err != nil {
@@ -375,7 +388,7 @@ func (r *Restore) prepareSnapshot() (err error) {
 }
 
 const (
-	preserveUUID = false
+	preserveUUID = true
 
 	batchSizeDefault           = 500
 	numInsertionWorkersDefault = 10
@@ -443,16 +456,12 @@ func (r *Restore) RunSnapshot() (err error) {
 		numInsertionWorkers = cfg.Restore.NumInsertionWorkers
 	}
 
-	defer func() {
-		err := r.node.DropTMPcoll()
-		if err != nil {
-			r.log.Error("dropping tmp collections: %v", err)
-		}
-	}()
-
 	mr := mongorestore.MongoRestore{
 		SessionProvider: rsession,
 		ToolOptions:     &topts,
+		// Have to handle users and roles on our own.
+		// see: https://jira.percona.com/browse/PBM-636 and comments.
+		SkipUsersAndRoles: true,
 		InputOptions: &mongorestore.InputOptions{
 			Archive: "-",
 		},
@@ -464,14 +473,10 @@ func (r *Restore) RunSnapshot() (err error) {
 			NumParallelCollections:   1,
 			PreserveUUID:             preserveUUID,
 			StopOnError:              true,
-			TempRolesColl:            "temproles",
-			TempUsersColl:            "tempusers",
 			WriteConcern:             "majority",
 		},
 		NSOptions: &mongorestore.NSOptions{
 			NSExclude: excludeFromRestore,
-			NSFrom:    []string{`admin.system.users`, `admin.system.roles`},
-			NSTo:      []string{pbm.DB + `.` + pbm.TmpUsersCollection, pbm.DB + `.` + pbm.TmpRolesCollection},
 		},
 		InputReader: dumpReader,
 	}
@@ -500,6 +505,24 @@ func (r *Restore) RunSnapshot() (err error) {
 		return errors.Wrap(err, "waiting for start")
 	}
 
+	r.log.Info("restoring users and roles")
+	cusr, err := r.node.CurrentUser()
+	if err != nil {
+		return errors.Wrap(err, "get current user")
+	}
+
+	err = r.swapUsers(r.cn.Context(), cusr)
+	if err != nil {
+		return errors.Wrap(err, "swap users 'n' roles")
+	}
+
+	defer func() {
+		err = pbm.DropTMPcoll(r.cn.Context(), r.node.Session())
+		if err != nil {
+			r.log.Warning("drop tmp collections: %v", err)
+		}
+	}()
+
 	r.log.Info("starting oplog replay")
 
 	or, err := r.stg.SourceReader(r.oplogFile)
@@ -520,17 +543,6 @@ func (r *Restore) RunSnapshot() (err error) {
 	}
 	r.log.Info("oplog replay finished on %v", lts)
 
-	cusr, err := r.node.CurrentUser()
-	if err != nil {
-		return errors.Wrap(err, "get current user")
-	}
-
-	r.log.Info("restoring users and roles")
-	err = r.restoreUsers(cusr)
-	if err != nil {
-		return errors.Wrap(err, "restore users 'n' roles")
-	}
-
 	return nil
 }
 
@@ -651,10 +663,6 @@ func (r *Restore) swapUsers(ctx context.Context, exclude *pbm.AuthInfo) error {
 	return nil
 }
 
-func (r *Restore) restoreUsers(exclude *pbm.AuthInfo) error {
-	return r.swapUsers(r.cn.Context(), exclude)
-}
-
 func (r *Restore) reconcileStatus(status pbm.Status, timeout *time.Duration) error {
 	if timeout != nil {
 		return errors.Wrap(r.convergeClusterWithTimeout(status, *timeout), "convergeClusterWithTimeout")
